"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const util_1 = require("util");
const os_1 = require("os");
const path_1 = require("path");
const js_yaml_1 = require("js-yaml");
const Debug = require("debug");
const debug = Debug('@fireblink/k8s-api-client');
class KubeConfig {
    /**
     * Load configuration from all posible sources
     */
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            // load based on KUBECONFIG env var value
            /* istanbul ignore else */
            if (process.env.KUBECONFIG && process.env.KUBECONFIG.length > 0) {
                debug(`Loading kube configuration based on KUBECONFIG evn var: ${process.env.KUBECONFIG}`);
                yield this.loadFromFile(process.env.KUBECONFIG);
                return;
            }
            // load based on default config file
            const defaultKubeConfig = path_1.join(os_1.homedir(), '.kube', 'config');
            const defaultKubeConfigExists = yield util_1.promisify(fs_1.exists)(defaultKubeConfig);
            /* istanbul ignore else */
            if (defaultKubeConfigExists) {
                debug(`Loading kube configuration from default location: ${defaultKubeConfig}`);
                yield this.loadFromFile(defaultKubeConfig);
                return;
            }
            // load on pod (from serviceaccount folder)
            const tokenExists = yield util_1.promisify(fs_1.exists)(KubeConfig.SA_TOKEN_PATH);
            /* istanbul ignore else */
            if (tokenExists) {
                debug(`Loading kube configuration from service account.`);
                yield this.loadFromServiceAccountToken();
                return;
            }
            throw new Error('Unable to load kube config');
        });
    }
    /**
     * Load configuration from pod
     */
    loadFromServiceAccountToken() {
        return __awaiter(this, void 0, void 0, function* () {
            const host = process.env.KUBERNETES_SERVICE_HOST;
            const port = process.env.KUBERNETES_SERVICE_PORT;
            let scheme = 'https';
            if (port === '80' || port === '8080' || port === '8001') {
                scheme = 'http';
            }
            const token = yield util_1.promisify(fs_1.readFile)(KubeConfig.SA_TOKEN_PATH, 'utf8');
            debug(`token: ${token}`);
            this.user = {
                name: '',
                user: {
                    token,
                },
            };
            debug(`user: ${JSON.stringify(this.user)}`);
            this.cluster = {
                name: '',
                cluster: {
                    server: `${scheme}://${host}:${port}`,
                    'certificate-authority': KubeConfig.SA_CA_PATH,
                },
            };
            debug(`cluster: ${JSON.stringify(this.cluster)}`);
        });
    }
    /**
     * Load KubeConfig from file
     * @param path
     * @param currentContext override `current-context` value with custom one
     */
    loadFromFile(path, currentContext) {
        return __awaiter(this, void 0, void 0, function* () {
            debug(`Loading kube configuration from file: ${path}`);
            const yaml = yield util_1.promisify(fs_1.readFile)(path, 'utf8');
            const obj = js_yaml_1.safeLoad(yaml);
            if (obj.apiVersion !== 'v1') {
                throw new TypeError(`Unable to load config at path: ${path}; unknown api version specified: ${obj.apiVersion}.`);
            }
            currentContext = currentContext || obj['current-context'];
            debug(`current-context: ${currentContext}`);
            if (!currentContext) {
                throw new Error(`Unable to load config at path: ${path}; no current-context field specified.`);
            }
            if (!obj.clusters) {
                throw new Error(`Unable to load config at path: ${path}; no clusters field specified.`);
            }
            if (!obj.contexts) {
                throw new Error(`Unable to load config at path: ${path}; no contexts field specified.`);
            }
            if (!obj.users) {
                throw new Error(`Unable to load config at path: ${path}; no users field specified.`);
            }
            const context = obj.contexts.find((c) => c.name === currentContext);
            if (!context) {
                throw new Error(`Unable to load config at path: ${path}; no matching context for name: ${currentContext}.`);
            }
            this.user = obj.users.find((u) => u.name === context.context.user);
            debug(`user: ${JSON.stringify(this.user)}`);
            if (!this.user) {
                throw new Error(`Unable to load config at path: ${path}; no matching user for name: ${context.context.user}.`);
            }
            this.cluster = obj.clusters.find((c) => c.name === context.context.cluster);
            debug(`cluster: ${JSON.stringify(this.user)}`);
            if (!this.cluster) {
                throw new Error(`Unable to load config at path: ${path}; no matching cluster for name: ${context.context.cluster}.`);
            }
        });
    }
}
KubeConfig.SA_ROOT = '/var/run/secrets/kubernetes.io/serviceaccount';
KubeConfig.SA_CA_PATH = KubeConfig.SA_ROOT + '/ca.crt';
KubeConfig.SA_TOKEN_PATH = KubeConfig.SA_ROOT + '/token';
exports.KubeConfig = KubeConfig;
//# sourceMappingURL=KubeConfig.js.map