"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request");
const BaseRequestProcessor_1 = require("./BaseRequestProcessor");
const Debug = require("debug");
const models_1 = require("../models");
const debug = Debug('@fireblink/k8s-api-client');
class APIRequestProcessor extends BaseRequestProcessor_1.BaseRequestProcessor {
    /**
     * Make request
     * @param options
     */
    makeRequest(options) {
        return __awaiter(this, void 0, void 0, function* () {
            options.json = true;
            if (debug.enabled) {
                debug(`Making ${options.method} request for URL: ${options.url}`);
                if (options.qs) {
                    debug(`Requer query parameters: ${JSON.stringify(options.qs)}`);
                }
                if (options.body) {
                    debug(`Request body: ${JSON.stringify(options.qs)}`);
                }
            }
            yield this.updateRequestOptions(options);
            return yield new Promise((resolve, reject) => {
                request(options, (error, response, body) => {
                    if (error) {
                        reject(error);
                    }
                    else {
                        if (debug.enabled) {
                            debug(`Response received. Status code: ${response.statusCode} Body: ${JSON.stringify(body)}`);
                        }
                        if (response.statusCode >= 200 && response.statusCode <= 299) {
                            resolve(body);
                        }
                        else {
                            reject(new models_1.RequestError(`${options.method} ${options.url} request failed. ${response.statusCode}: ${response.statusMessage}`, response, body));
                        }
                    }
                });
            });
        });
    }
    /**
     * Make GET request
     * @param path
     * @param [queryParameters]
     */
    get(path, queryParameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const kc = yield this.loadConfig();
            const options = {
                method: 'GET',
                url: kc.cluster.cluster.server + path,
                qs: queryParameters,
            };
            return yield this.makeRequest(options);
        });
    }
    /**
     * Make POST request
     * @param path
     * @param body
     * @param [queryParameters]
     */
    post(path, body, queryParameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const kc = yield this.loadConfig();
            const options = {
                method: 'POST',
                url: kc.cluster.cluster.server + path,
                qs: queryParameters,
                body: body,
            };
            return yield this.makeRequest(options);
        });
    }
    /**
     * Make PUT request
     * @param path
     * @param body
     * @param [queryParameters]
     */
    put(path, body, queryParameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const kc = yield this.loadConfig();
            const options = {
                method: 'PUT',
                url: kc.cluster.cluster.server + path,
                qs: queryParameters,
                body: body,
            };
            return yield this.makeRequest(options);
        });
    }
    /**
     * Make PATCH request (json patch).
     * If you want to just simple merge objects - use `merge` method instead.
     * @see https://tools.ietf.org/html/rfc6902
     * @param path
     * @param body
     * @param [queryParameters]
     */
    patch(path, body, queryParameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const kc = yield this.loadConfig();
            const options = {
                method: 'PATCH',
                url: kc.cluster.cluster.server + path,
                qs: queryParameters,
                headers: {
                    'content-type': 'application/json-patch+json',
                },
                body: body,
            };
            return yield this.makeRequest(options);
        });
    }
    /**
     * Make PATCH request (json merge)
     * @see https://tools.ietf.org/html/rfc7386
     * @param path
     * @param body
     * @param [queryParameters]
     */
    merge(path, body, queryParameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const kc = yield this.loadConfig();
            const options = {
                method: 'PATCH',
                url: kc.cluster.cluster.server + path,
                qs: queryParameters,
                headers: {
                    'content-type': 'application/merge-patch+json',
                },
                body: body,
            };
            return yield this.makeRequest(options);
        });
    }
    /**
     * Make PATCH request
     * @param path
     * @param [queryParameters]
     */
    delete(path, queryParameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const kc = yield this.loadConfig();
            const options = {
                method: 'DELETE',
                url: kc.cluster.cluster.server + path,
                qs: queryParameters,
            };
            return yield this.makeRequest(options);
        });
    }
    /**
     * Make k8s GET requests sequence to fetch all records
     * @param path
     * @param queryParameters
     */
    getAll(path, queryParameters) {
        return __awaiter(this, void 0, void 0, function* () {
            debug(`GET request processor: getting all items for path: ${path} and queryParameters: ${queryParameters}`);
            if (!queryParameters) {
                queryParameters = {};
            }
            if (!queryParameters.limit) {
                queryParameters.limit = 100;
            }
            const result = {
                resourceVersion: '',
                items: [],
            };
            let continueValue;
            let response;
            do {
                queryParameters.continue = continueValue;
                response = yield this.get(path, queryParameters);
                result.items.push(...response.items);
                result.resourceVersion = response.metadata.resourceVersion;
                continueValue = response.metadata.continue;
            } while (continueValue && continueValue.length);
            return result;
        });
    }
}
exports.APIRequestProcessor = APIRequestProcessor;
//# sourceMappingURL=APIRequestProcessor.js.map