"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request");
const json_streaming_reader_1 = require("json-streaming-reader");
const BaseRequestProcessor_1 = require("./BaseRequestProcessor");
const Debug = require("debug");
const models_1 = require("../models");
const debug = Debug('@fireblink/k8s-api-client');
class WatchRequestProcessor extends BaseRequestProcessor_1.BaseRequestProcessor {
    constructor() {
        super(...arguments);
        this.aborted = false;
    }
    /**
     * Abort active watch request
     */
    abort() {
        this.aborted = true;
        if (this.activeRequest) {
            this.activeRequest.abort();
            this.activeRequest = undefined;
        }
    }
    /**
     * Establish long polling request to monitor k8s resources
     * @param path
     * @param handlers
     * @param resourceVersion
     */
    makeWatchRequest(path, handle) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.aborted) {
                throw new Error('Unable to watch. Processor already aborted.');
            }
            debug(`WATCH request processor - start watching path: ${path} and resourceVersion: ${this.resourseVersion}`);
            const kc = yield this.loadConfig();
            const options = {
                url: kc.cluster.cluster.server + path,
                method: 'GET',
                headers: {},
                qs: {},
            };
            yield this.updateRequestOptions(options);
            options.qs.watch = 'true';
            if (this.resourseVersion) {
                options.qs.resourceVersion = this.resourseVersion;
            }
            this.activeRequest = undefined;
            try {
                yield new Promise((res, rej) => {
                    const stream = request(options);
                    const jsonStream = new json_streaming_reader_1.JsonStreamReader();
                    stream.pipe(jsonStream);
                    stream.on('error', err => {
                        this.abort();
                        rej(err);
                    });
                    jsonStream.on('data', (data) => {
                        const record = data.record;
                        if (record.kind && record.kind.toLowerCase() === 'status') {
                            if (record.status && record.status.toLowerCase() === 'failure') {
                                jsonStream.pause();
                                this.abort();
                                rej(new Error(record.message));
                                return;
                            }
                        }
                        if (record.type) {
                            jsonStream.pause();
                            this.resourseVersion = record.object.metadata.resourceVersion;
                            handle(record).then(() => {
                                jsonStream.resume();
                            }, err => {
                                this.abort();
                                rej(err);
                            });
                        }
                    });
                    stream.on('request', req => {
                        this.activeRequest = req;
                    });
                    let response;
                    stream.on('response', resp => {
                        debug(`WATCH request processor - received response for path: ${path}`);
                        response = resp;
                    });
                    jsonStream.on('finish', () => {
                        debug(`WATCH request processor - finished watching for path: ${path}`);
                        if (!response) {
                            rej(new models_1.RequestError('Request failed'));
                            return;
                        }
                        if (response.statusCode >= 400) {
                            rej(new models_1.RequestError('Request failed', response));
                            return;
                        }
                        res();
                    });
                });
            }
            finally {
                debug(`WATCH request processor - finished watching path: ${path}`);
                this.activeRequest = undefined;
            }
        });
    }
    /**
     * Establish long polling request to monitor k8s resources
     * @param path
     * @param handlers
     * @param resourceVersion
     */
    watch(path, handlers, resourceVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            this.resourseVersion = resourceVersion;
            const handle = (record) => __awaiter(this, void 0, void 0, function* () {
                const type = record.type.toLowerCase();
                debug(`Received record with type: ${type}`);
                if (type === 'added' || type === 'modified' || type === 'deleted') {
                    const handler = handlers[type];
                    if (handler) {
                        yield handler(record.object);
                    }
                }
            });
            while (!this.aborted) {
                yield this.makeWatchRequest(path, handle);
            }
        });
    }
}
exports.WatchRequestProcessor = WatchRequestProcessor;
//# sourceMappingURL=WatchRequestProcessor.js.map